# -*- coding: UTF-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

"""
GUI test tool and automation framework that uses Accessibility (a11y) technologies to communicate
with desktop applications.
"""

__author__ = """Zack Cerza <zcerza@redhat.com>,
Ed Rousseau <rousseau@redhat.com>,
David Malcolm <dmalcolm@redhat.com>,
Vita Humpa <vhumpa@redhat.com>"""
__version__ = "0.9.11"
__copyright__ = "Copyright © 2005-2017 Red Hat, Inc."
__license__ = "GPL"

__all__ = ("config", "distro", "dump", "errors", "i18n", "logging", "path", "predicate",
           "procedural", "rawinput", "sessions", "tc", "tree", "utils", "version", "wrapped")
